==> fbsdmon-0.93.tar.gz: fbsdmon-0.93/src/uuid.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sha256.h>
#include <curl/curl.h>

#include "io.h"


extern char *uuid_gen()
{
    SHA256_CTX ctx;
    char buff[65];

    char *sysid = uname("-a");

    char *rand;
    srandomdev();
    asprintf(&rand, "%ld\n", random());

    char *now;
    asprintf(&now, "%ld", time(NULL));

    SHA256_Init(&ctx);
    SHA256_Update(&ctx, "FbsdMon", 7);
    SHA256_Update(&ctx, sysid, strlen(sysid));
    SHA256_Update(&ctx, rand, strlen(rand));
    SHA256_Update(&ctx, now, strlen(now));
    SHA256_End(&ctx, buff);

    free(sysid);
    free(rand);
    free(now);

    return strdup(buff);
}

char *uuid_path()
{
    char *fp;
    asprintf(&fp, "/var/db/fbsdmon/ID");
    return fp;
}

int uuid_dir_exist()
{
    struct stat sb;
    char *dp = uuid_path();
    dp[strlen(dp) - 3] = '\0';

    stat(dp, &sb);
    free(dp);

    return S_ISDIR(sb.st_mode);
}

extern char *uuid_read()
{
    char *fp = uuid_path();
    FILE *fh;
    char uuid[65] = "";

    if(NULL != (fh = fopen(fp, "r"))) {
        fgets(uuid, 65, fh);
        fclose(fh);
    }

    free(fp);

    return '\0' != uuid[0] ? strdup(uuid) : NULL;
}

extern void uuid_write(char *uuid)
{
    char *fp = uuid_path();
    FILE *fh;

    if(NULL != (fh = fopen(fp, "w"))) {
        fputs(uuid, fh);
        fclose(fh);
    }

    free(fp);
}

extern int uuid_reg(char *uuid)
{
    CURL *curl = curl_easy_init();
    char *url = fbsdmon_url("uuid");
    char *ua = useragent();
    char *post_data;

    curl_easy_setopt(curl, CURLOPT_USERAGENT, ua);
    curl_easy_setopt(curl, CURLOPT_URL, url);

    asprintf(&post_data, "reg=%s", uuid);
    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, post_data);

    if(NULL != getenv("FBSDMON_URL")) {
        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
    }

    long status_code;
    curl_easy_perform(curl);
    curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &status_code);

    curl_easy_cleanup(curl);
    free(ua);
    free(url);
    free(post_data);

    return status_code;
}

extern char *uuid()
{
    char *uuid;

    if(! uuid_dir_exist()) {
        puts("/var/db/fbsdmon directory don't exist");
        exit(1);
    }

    uuid = uuid_read();

    if(NULL == uuid)
    {
        while(1)
        {
            int status_code;

            uuid = uuid_gen();
            status_code = uuid_reg(uuid);

            if(201 == status_code) break;
            else if(409 == status_code) continue;
            else {
                printf("%d: server error\n", status_code);
                exit(1);
            }
        }

        uuid_write(uuid);
    }

    return uuid;
}

==> fbsdmon-1.00.tar.gz: fbsdmon-1.00/src/uuid.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sha256.h>
#include <curl/curl.h>

#include "io.h"


extern char *uuid_gen()
{
    SHA256_CTX ctx;
    char buff[65];

    char *sysid = uname("-a");

    char *rand;
    srandomdev();
    asprintf(&rand, "%ld\n", random());

    char *now;
    asprintf(&now, "%d", time(NULL));

    SHA256_Init(&ctx);
    SHA256_Update(&ctx, "FbsdMon", 7);
    SHA256_Update(&ctx, sysid, strlen(sysid));
    SHA256_Update(&ctx, rand, strlen(rand));
    SHA256_Update(&ctx, now, strlen(now));
    SHA256_End(&ctx, buff);

    free(sysid);
    free(rand);
    free(now);

    return strdup(buff);
}

char *uuid_path()
{
    char *fp;
    asprintf(&fp, "/var/db/fbsdmon/ID");
    return fp;
}

int uuid_dir_exist()
{
    struct stat sb;
    char *dp = uuid_path();
    dp[strlen(dp) - 3] = '\0';

    stat(dp, &sb);
    free(dp);

    return S_ISDIR(sb.st_mode);
}

extern char *uuid_read()
{
    char *fp = uuid_path();
    FILE *fh;
    char uuid[65] = "";

    if(NULL != (fh = fopen(fp, "r"))) {
        fgets(uuid, 65, fh);
        fclose(fh);
    }

    free(fp);

    return '\0' != uuid[0] ? strdup(uuid) : NULL;
}

extern void uuid_write(char *uuid)
{
    char *fp = uuid_path();
    FILE *fh;

    if(NULL != (fh = fopen(fp, "w"))) {
        fputs(uuid, fh);
        fclose(fh);
    }

    free(fp);
}

extern int uuid_reg(char *uuid)
{
    CURL *curl = curl_easy_init();
    char *url = fbsdmon_url("uuid");
    char *ua = useragent();
    char *post_data;

    curl_easy_setopt(curl, CURLOPT_USERAGENT, ua);
    curl_easy_setopt(curl, CURLOPT_URL, url);

    asprintf(&post_data, "reg=%s", uuid);
    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, post_data);

    if(NULL != getenv("FBSDMON_URL")) {
        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
    }

    long status_code;
    curl_easy_perform(curl);
    curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &status_code);

    curl_easy_cleanup(curl);
    free(ua);
    free(url);
    free(post_data);

    return status_code;
}

extern char *uuid()
{
    char *uuid;

    if(! uuid_dir_exist()) {
        puts("/var/db/fbsdmon directory don't exist");
        exit(1);
    }

    uuid = uuid_read();

    if(NULL == uuid)
    {
        while(1)
        {
            int status_code;

            uuid = uuid_gen();
            status_code = uuid_reg(uuid);

            if(201 == status_code) break;
            else if(409 == status_code) continue;
            else {
                printf("%d: server error\n", status_code);
                exit(1);
            }
        }

        uuid_write(uuid);
    }

    return uuid;
}
